package com.laihua.projecthelper.stringxml

import com.laihua.projecthelper.code.Module
import com.laihua.projecthelper.implement.belongModule
import com.laihua.projecthelper.util.exeCmd
import org.jdom2.Document
import org.jdom2.Element
import org.jdom2.input.SAXBuilder
import org.jdom2.output.Format
import org.jdom2.output.XMLOutputter
import java.io.File
import java.io.FileWriter


/**
 * Author: pengjunjing
 * Date: 2022/3/11
 * Description:string的xml文件解析类
 */
class StringXmlParse {

    companion object {

        const val resourcesStringDef = "resources"
        const val nameStringDef = "name"
        const val stringKeyDef = "string"

    }

    /**
     * 将字符串文件解析成对象
     */
    fun parseToObj(stringXmlFile: File): StringXmlResCode {
        val document = SAXBuilder().build(stringXmlFile)
        val rootElement = document.rootElement
        val belongModule = stringXmlFile.belongModule()
        val stringXmlResCode = StringXmlResCode(stringXmlFile, belongModule)
        //读取resources节点下的内容
        if (belongModule != null && resourcesStringDef == rootElement.name) {
            val children = rootElement.children
            for (child in children) {
                if (child.name == stringKeyDef) {
                    //获取string节点的数据
                    val stringElement =
                        StringElement(
                            stringXmlFile,
                            belongModule,
                            child.attributes[0].value,
                            child.value
                        )
                    stringXmlResCode.stringElementList.add(stringElement)
                }
            }

        }
        return stringXmlResCode
    }

    /**
     * 删除掉一个xml文件字符串内容
     */
    fun deleteStringXmlElement(stringElement: StringElement) {
        val document = SAXBuilder().build(stringElement.file)
        val rootElement = document.rootElement
        //找到与name=xxx相同的值
        val find: Element? = rootElement.children.find {
            it.getAttribute(nameStringDef).value == stringElement.attributeName
        }
        find?.let {
            rootElement.removeContent(it)
        }

        saveXML(stringElement.file, document)
    }

    private fun saveXML(file: File, doc: Document): Boolean {
        // 将doc对象输出到文件
        try {
            // 创建xml文件输出流
            val xmlOutPutter = XMLOutputter()

            // 创建文件输出流
            val writer = FileWriter(file)

            // 指定文档格式
            val fm: Format = Format.getPrettyFormat()
            xmlOutPutter.format = fm

            // 将doc写入到指定的文件中
            xmlOutPutter.output(doc, writer)
            writer.close()
            return true
        } catch (e: Exception) {
            e.printStackTrace()
            return false
        }
    }

    /**
     * 移动一个string的定义到指定的模块下
     */
    fun moveStringXmlElement(stringElement: StringElement, targetModule: Module) {
        //目标xml的路径: module/src/main/res/values
        val stringXmlPath = stringElement.file.absolutePath.replace(
            stringElement.belongModule,
            targetModule.moduleName
        )
//        val stringXmlPath = "${targetModule.moduleFile}" +
//                "${File.pathSeparator}src${File.pathSeparator}main" +
//                "${File.pathSeparator}res${File.pathSeparator}value" +
//                "${File.pathSeparator}strings.xml"
        val targetFile = File(stringXmlPath)
        //判断创建文件
        createFile(targetFile)
        val document = SAXBuilder().build(targetFile)
        if (!document.hasRootElement()) {
            //创建根节点
            document.rootElement = Element(resourcesStringDef)
        }

        //查找该变量存不存在
        val findChild = document.rootElement.findChild(stringElement.attributeName)
        if (findChild == null) {
            //节点不存在,创建节点:    <string name="recharge">充值</string>
            val element = Element(stringKeyDef)
            element.setAttribute(nameStringDef, stringElement.attributeName)
            element.addContent(stringElement.value)
            //添加一个字符串定义
            document.rootElement.addContent(element)
            //保存
            if (saveXML(targetFile, document)) {
                //保存成功,删除源文件的字符串
                deleteStringXmlElement(stringElement)
            }
        } else {
            //节点已存在,不移动
            print("该值无法移动,目标位置已有同名字符串:${stringElement.attributeName} -> ${targetModule.moduleName}")
        }
    }

    /**
     * 判断一个文件存不存在,不存在则创建该文件
     */
    private fun createFile(targetFile: File) {
        //所在文件夹不存在则创建文件夹
        if (!targetFile.parentFile.exists()) {
            targetFile.parentFile.mkdirs()
        }
        if (!targetFile.exists()) {
            if (targetFile.createNewFile()) {
                //写入初始的xml文件
                targetFile.writeText( "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n" +
                        "<resources>\n" +
                        "" +
                        "</resources>")
                //添加到git的版本控制
                exeCmd("git add .")
            }
        }
    }

    private fun Element.findChild(nameAttributeValue: String): Element? {
        for (child in this.children) {
            for (attribute in child.attributes) {
                if (attribute.name == nameStringDef && attribute.value == nameAttributeValue) {
                    return child
                }
            }
        }
        return null
    }

}